@@ -1,12 +1,12 @@
 /*                     __                                               *\
 **     ________ ___   / /  ___     Scala API                            **
-**    / __/ __// _ | / /  / _ |    (c) 2003-2008, LAMP/EPFL             **
+**    / __/ __// _ | / /  / _ |    (c) 2003-2009, LAMP/EPFL             **
 **  __\ \/ /__/ __ |/ /__/ __ |    http://scala-lang.org/               **
 ** /____/\___/_/ |_/____/_/ | |                                         **
 **                          |/                                          **
 \*                                                                      */
 
-// $Id: Seq.scala 16059 2008-09-08 13:07:31Z washburn $
+// $Id: Seq.scala 17137 2009-02-17 20:09:49Z rytz $
 
 
 package scala
@@ -470,15 +470,8 @@ trait Seq[+A] extends AnyRef with PartialFunction[Int, A] with Collection[A] {
   /** @return true if this sequence end with that sequence 
    *  @see String.endsWith 
    */
-  def endsWith[B](that: Seq[B]): Boolean = {
-    val length = this.length
-    val j = that.elements
-    var i = 0
-    var result = true
-    while (result && i < length && j.hasNext) 
-      result = apply(length - i - 1) == j.next
-    result && !j.hasNext
-  }
+  def endsWith[B](that: Seq[B]): Boolean =
+    drop(length - that.length).sameElements(that)
 
   /**
    * Searches for the argument sequence in the receiver object, returning 
