@@ -1,32 +1,35 @@
 /*                     __                                               *\
 **     ________ ___   / /  ___     Scala API                            **
-**    / __/ __// _ | / /  / _ |    (c) 2005-2006, LAMP/EPFL             **
-**  __\ \/ /__/ __ |/ /__/ __ |                                         **
+**    / __/ __// _ | / /  / _ |    (c) 2005-2009, LAMP/EPFL             **
+**  __\ \/ /__/ __ |/ /__/ __ |    http://scala-lang.org/               **
 ** /____/\___/_/ |_/____/_/ | |                                         **
 **                          |/                                          **
 \*                                                                      */
 
-// $Id: Actor.scala 10032 2007-02-19 10:45:23Z phaller $
+// $Id: Actor.scala 17006 2009-01-30 17:02:45Z phaller $
 
 package scala.actors
 
 import scala.collection.mutable.{HashSet, Queue}
-import compat.Platform
+import scala.compat.Platform
+
+import java.util.{Timer, TimerTask}
 
 /**
  * The <code>Actor</code> object provides functions for the definition of
- * actors, as well as all actor operations, such as
+ * actors, as well as actor operations, such as
  * <code>receive</code>, <code>react</code>, <code>reply</code>,
  * etc.
  *
- * @version 0.9.4
+ * @version 0.9.18
  * @author Philipp Haller
  */
 object Actor {
 
-  //private[actors] val selfs = new java.util.WeakHashMap(16, 0.5f)
+  private[actors] val tl = new ThreadLocal[Actor]
 
-  private[actors] val tl = new ThreadLocal
+  // timer thread runs as daemon
+  private[actors] val timer = new Timer(true)
 
   /**
    * Returns the currently executing actor. Should be used instead
@@ -35,25 +38,48 @@ object Actor {
    *
    * @return returns the currently executing actor.
    */
-  def self: Actor = synchronized {
+  def self: Actor = {
     var a = tl.get.asInstanceOf[Actor]
     if (null eq a) {
       a = new ActorProxy(currentThread)
       tl.set(a)
     }
     a
-    /*val t = currentThread
-    var a = selfs.get(t).asInstanceOf[Actor]
-    if (a eq null) {
-      a = new ActorProxy(t)
-      selfs.put(t, a)
-    }
-    a*/
   }
 
   /**
-   * <p>This function is used for the definition of actors.</p>
-   * <p>The following example demonstrates its usage:</p><pre>
+   * Resets an actor proxy associated with the current thread.
+   * It replaces the implicit <code>ActorProxy</code> instance
+   * of the current thread (if any) with a new instance.
+   *
+   * This permits to re-use the current thread as an actor
+   * even if its <code>ActorProxy</code> has died for some reason.
+   */
+  def resetProxy {
+    val a = tl.get.asInstanceOf[Actor]
+    if ((null ne a) && a.isInstanceOf[ActorProxy])
+      tl.set(new ActorProxy(currentThread))
+  }
+
+  /**
+   * Removes any reference to an <code>Actor</code> instance
+   * currently stored in thread-local storage.
+   *
+   * This allows to release references from threads that are
+   * potentially long-running or being re-used (e.g. inside
+   * a thread pool). Permanent references in thread-local storage
+   * are a potential memory leak.
+   */
+  def clearSelf {
+    tl.set(null)
+  }
+
+  /**
+   * <p>This is a factory method for creating actors.</p>
+   *
+   * <p>The following example demonstrates its usage:</p>
+   *
+   * <pre>
    * import scala.actors.Actor._
    * ...
    * val a = actor {
@@ -64,7 +90,7 @@ object Actor {
    * @param  body  the code block to be executed by the newly created actor
    * @return       the newly created actor. Note that it is automatically started.
    */
-  def actor(body: => Unit): Actor = synchronized {
+  def actor(body: => Unit): Actor = {
     val actor = new Actor {
       def act() = body
     }
@@ -73,20 +99,37 @@ object Actor {
   }
 
   /**
-   * Creates an instance of a thread-based actor specifying a
-   * channel which can be used for typed communication with other
-   * actors.
+   * <p>
+   * This is a factory method for creating actors whose
+   * body is defined using a <code>Responder</code>.
+   * </p>
+   *
+   * <p>The following example demonstrates its usage:</p>
+   *
+   * <pre>
+   * import scala.actors.Actor._
+   * import Responder.exec
+   * ...
+   * val a = reactor {
+   *   for {
+   *     res <- b !! MyRequest;
+   *     if exec(println("result: "+res))
+   *   } yield {}
+   * }
+   * </pre>
+   *
+   * @param  body  the <code>Responder</code> to be executed by the newly created actor
+   * @return       the newly created actor. Note that it is automatically started.
    */
-/*
-  def actor[a](ch: Channel[a])(body: => Unit): Actor = synchronized {
-    val actor = new Actor {
-      def act() = body
+  def reactor(body: => Responder[Unit]): Actor = {
+    val a = new Actor {
+      def act() {
+        Responder.run(body)
+      }
     }
-    ch.receiver = actor
-    actor.start()
-    actor
+    a.start()
+    a
   }
-*/
 
   /**
    * Receives the next message from the mailbox of the current actor
@@ -102,7 +145,7 @@ object Actor {
    * @param  f a partial function specifying patterns and actions
    * @return   the result of processing the received message
    */
-  def receive[a](f: PartialFunction[Any, a]): a =
+  def receive[A](f: PartialFunction[Any, A]): A =
     self.receive(f)
 
   /**
@@ -117,7 +160,7 @@ object Actor {
    * @param  f    a partial function specifying patterns and actions
    * @return      the result of processing the received message
    */
-  def receiveWithin[R](msec: long)(f: PartialFunction[Any, R]): R =
+  def receiveWithin[R](msec: Long)(f: PartialFunction[Any, R]): R =
     self.receiveWithin(msec)(f)
 
   /**
@@ -144,7 +187,7 @@ object Actor {
    * @param  f    a partial function specifying patterns and actions
    * @return      this function never returns
    */
-  def reactWithin(msec: long)(f: PartialFunction[Any, Unit]): Nothing =
+  def reactWithin(msec: Long)(f: PartialFunction[Any, Unit]): Nothing =
     self.reactWithin(msec)(f)
 
   def eventloop(f: PartialFunction[Any, Unit]): Nothing =
@@ -152,9 +195,9 @@ object Actor {
 
   private class RecursiveProxyHandler(a: Actor, f: PartialFunction[Any, Unit])
           extends PartialFunction[Any, Unit] {
-    def isDefinedAt(m: Any): boolean =
+    def isDefinedAt(m: Any): Boolean =
       true // events are immediately removed from the mailbox
-    def apply(m: Any): Unit = {
+    def apply(m: Any) {
       if (f.isDefinedAt(m)) f(m)
       self.react(this)
     }
@@ -163,7 +206,7 @@ object Actor {
   /**
    * Returns the actor which sent the last received message.
    */
-  def sender: Actor = self.sender
+  def sender: OutputChannel[Any] = self.sender
 
   /**
    * Send <code>msg</code> to the actor waiting in a call to
@@ -177,12 +220,41 @@ object Actor {
    */
   def reply(): Unit = self.reply(())
 
+  /**
+   * Returns the number of messages in <code>self</code>'s mailbox
+   *
+   * @return the number of messages in <code>self</code>'s mailbox
+   */
+  def mailboxSize: Int = self.mailboxSize
+
+  /**
+   * <p>
+   * Converts a synchronous event-based operation into
+   * an asynchronous <code>Responder</code>.
+   * </p>
+   *
+   * <p>The following example demonstrates its usage:</p>
+   * 
+   * <pre>
+   * val adder = reactor {
+   *   for {
+   *     _ <- respondOn(react) { case Add(a, b) => reply(a+b) }
+   *   } yield {}
+   * }
+   * </pre>
+   */
+  def respondOn[A, B](fun: PartialFunction[A, Unit] => Nothing):
+    PartialFunction[A, B] => Responder[B] =
+      (caseBlock: PartialFunction[A, B]) => new Responder[B] {
+        def respond(k: B => Unit) = fun(caseBlock andThen k)
+      }
+
   private[actors] trait Body[a] {
-    def andThen[b](other: => b): Nothing
+    def andThen[b](other: => b): Unit
   }
 
   implicit def mkBody[a](body: => a) = new Body[a] {
-    def andThen[b](other: => b): Nothing = seq(body, other)
+    def andThen[b](other: => b): Unit = self.seq(body, other)
   }
 
   /**
@@ -191,22 +263,19 @@ object Actor {
    *
    * @param body the code block to be executed
    */
-  def loop(body: => Unit): Nothing = body andThen loop(body)
+  def loop(body: => Unit): Unit = body andThen loop(body)
 
   /**
-   * Causes <code>self</code> to execute <code>first</code>
-   * followed by <code>next</code>.
+   * Causes <code>self</code> to repeatedly execute
+   * <code>body</code> while the condition
+   * <code>cond</code> is <code>true</code>.
    *
-   * @param first the first code block to be executed
-   * @param next  the second code block to be executed
+   * @param cond the condition to test
+   * @param body the code block to be executed
    */
-  def seq[a, b](first: => a, next: => b): Nothing = {
-    val s = self
-    val killNext = s.kill
-    s.kill = () => { s.kill = killNext; next; exit('normal) }
-    first
-    exit('normal)
-  }
+  def loopWhile(cond: => Boolean)(body: => Unit): Unit =
+    if (cond) { body andThen loopWhile(cond)(body) }
+    else continue
 
   /**
    * Links <code>self</code> to actor <code>to</code>.
@@ -214,7 +283,7 @@ object Actor {
    * @param  to the actor to link to
    * @return 
    */
-  def link(to: Actor): Actor = self.link(to)
+  def link(to: AbstractActor): AbstractActor = self.link(to)
 
   /**
    * Links <code>self</code> to actor defined by <code>body</code>.
@@ -239,7 +308,7 @@ object Actor {
    * <p>
    *   For each linked actor <code>a</code> with
    *   <code>trapExit</code> set to <code>true</code>, send message
-   *   <code>{'EXIT, self, reason}</code> to <code>a</code>.
+   *   <code>Exit(self, reason)</code> to <code>a</code>.
    * </p>
    * <p>
    *   For each linked actor <code>a</code> with
@@ -249,63 +318,112 @@ object Actor {
    * </p>
    */
   def exit(reason: AnyRef): Nothing = self.exit(reason)
+
+  /**
+   * <p>
+   *   Terminates execution of <code>self</code> with the following
+   *   effect on linked actors:
+   * </p>
+   * <p>
+   *   For each linked actor <code>a</code> with
+   *   <code>trapExit</code> set to <code>true</code>, send message
+   *   <code>Exit(self, 'normal)</code> to <code>a</code>.
+   * </p>
+   */
+  def exit(): Nothing = self.exit()
+
+  def continue: Unit = throw new KillActorException
 }
 
 /**
  * <p>
- *   This class provides (together with <code>Channel</code>) an
- *   implementation of event-based actors.
- * </p>
- * <p>
- *   The main ideas of our approach are explained in the papers<br>
- *   <b>Event-Based Programming without Inversion of Control</b>,
- *   Philipp Haller and Martin Odersky, <i>Proc. JMLC 2006</i>
- *   <br><br>
- *   <b>Actors that Unify Threads and Events</b>,
- *   Philipp Haller and Martin Odersky, <i>LAMP-REPORT-2007-001, EPFL</i>
+ *   This class provides an implementation of event-based actors.
+ *   The main ideas of our approach are explained in the two papers
  * </p>
+ * <ul>
+ *   <li>
+ *     <a href="http://lampwww.epfl.ch/~odersky/papers/jmlc06.pdf">
+ *     <span style="font-weight:bold; white-space:nowrap;">Event-Based
+ *     Programming without Inversion of Control</span></a>,<br/>
+ *     Philipp Haller and Martin Odersky, <i>Proc. JMLC 2006</i>, and
+ *   </li>
+ *   <li>
+ *     <a href="http://lamp.epfl.ch/~phaller/doc/haller07coord.pdf">
+ *     <span style="font-weight:bold; white-space:nowrap;">Actors that
+ *     Unify Threads and Events</span></a>,<br/>
+ *     Philipp Haller and Martin Odersky, <i>Proc. COORDINATION 2007</i>.
+ *   </li>
+ * </ul>
  *
- * @version 0.9.4
+ * @version 0.9.18
  * @author Philipp Haller
  */
-trait Actor extends OutputChannel[Any] {
+@serializable
+trait Actor extends AbstractActor {
 
   private var received: Option[Any] = None
 
-  private[actors] val waitingForNone = (m: Any) => false
-  private[actors] var waitingFor: Any => boolean = waitingForNone
-  private[actors] var isSuspended = false
+  private val waitingForNone = (m: Any) => false
+  private var waitingFor: Any => Boolean = waitingForNone
+  private var isSuspended = false
 
-  private val mailbox = new MessageQueue
-  private var sessions: List[Channel[Any]] = Nil
+  protected val mailbox = new MessageQueue
+  private var sessions: List[OutputChannel[Any]] = Nil
 
-  private def send(msg: Any, session: Channel[Any]) = synchronized {
-    tick()
+  protected def scheduler: IScheduler =
+    Scheduler
+
+  /**
+   * Returns the number of messages in this actor's mailbox
+   *
+   * @return the number of messages in this actor's mailbox
+   */
+  def mailboxSize: Int = synchronized {
+    mailbox.size
+  }
+
+  /**
+   * Sends <code>msg</code> to this actor (asynchronous) supplying
+   * explicit reply destination.
+   *
+   * @param  msg      the message to send
+   * @param  replyTo  the reply destination
+   */
+  def send(msg: Any, replyTo: OutputChannel[Any]) = synchronized {
     if (waitingFor(msg)) {
       received = Some(msg)
-      sessions = session :: sessions
+
+      if (isSuspended)
+        sessions = replyTo :: sessions
+      else
+        sessions = List(replyTo)
+
       waitingFor = waitingForNone
 
-      if (timeoutPending) {
-        timeoutPending = false
-        TimerThread.trashRequest(this)
+      if (!onTimeout.isEmpty) {
+        onTimeout.get.cancel()
+        onTimeout = None
       }
 
       if (isSuspended)
         resumeActor()
-      else
-        scheduleActor(null, msg)
+      else // assert continuation != null
+        scheduler.execute(new Reaction(this, continuation, msg))
     } else {
-      mailbox.append(msg, session)
+      mailbox.append(msg, replyTo)
     }
   }
 
+  /**
+   * Receives a message from this actor's mailbox.
+   *
+   * @param  f    a partial function with message patterns and actions
+   * @return      result of processing the received value
+   */
   def receive[R](f: PartialFunction[Any, R]): R = {
     assert(Actor.self == this, "receive from channel belonging to other actor")
-    // links
-    if (shouldExit) exit()
+    if (shouldExit) exit() // links
     this.synchronized {
-      tick()
       val qel = mailbox.extractFirst((m: Any) => f.isDefinedAt(m))
       if (null eq qel) {
         waitingFor = f.isDefinedAt
@@ -323,12 +441,21 @@ trait Actor extends OutputChannel[Any] {
     result
   }
 
-  def receiveWithin[R](msec: long)(f: PartialFunction[Any, R]): R = {
+  /**
+   * Receives a message from this actor's mailbox within a certain
+   * time span.
+   *
+   * @param  msec the time span before timeout
+   * @param  f    a partial function with message patterns and actions
+   * @return      result of processing the received value
+   */
+  def receiveWithin[R](msec: Long)(f: PartialFunction[Any, R]): R = {
     assert(Actor.self == this, "receive from channel belonging to other actor")
-    // links
-    if (shouldExit) exit()
+    if (shouldExit) exit() // links
     this.synchronized {
-      tick()
+      // first, remove spurious TIMEOUT message from mailbox if any
+      val spurious = mailbox.extractFirst((m: Any) => m == TIMEOUT)
+
       val qel = mailbox.extractFirst((m: Any) => f.isDefinedAt(m))
       if (null eq qel) {
         if (msec == 0) {
@@ -365,42 +492,70 @@ trait Actor extends OutputChannel[Any] {
     result
   }
 
+  /**
+   * Receives a message from this actor's mailbox.
+   * <p>
+   * This method never returns. Therefore, the rest of the computation
+   * has to be contained in the actions of the partial function.
+   *
+   * @param  f    a partial function with message patterns and actions
+   */
   def react(f: PartialFunction[Any, Unit]): Nothing = {
     assert(Actor.self == this, "react on channel belonging to other actor")
-    // links
-    if (shouldExit) exit()
-    Scheduler.pendReaction
+    if (shouldExit) exit() // links
     this.synchronized {
-      tick()
       val qel = mailbox.extractFirst((m: Any) => f.isDefinedAt(m))
       if (null eq qel) {
         waitingFor = f.isDefinedAt
         continuation = f
         isDetached = true
       } else {
-        sessions = qel.session :: sessions
+        sessions = List(qel.session)
         scheduleActor(f, qel.msg)
       }
       throw new SuspendActorException
     }
   }
 
-  def reactWithin(msec: long)(f: PartialFunction[Any, Unit]): Nothing = {
+  /**
+   * Receives a message from this actor's mailbox within a certain
+   * time span.
+   * <p>
+   * This method never returns. Therefore, the rest of the computation
+   * has to be contained in the actions of the partial function.
+   *
+   * @param  msec the time span before timeout
+   * @param  f    a partial function with message patterns and actions
+   */
+  def reactWithin(msec: Long)(f: PartialFunction[Any, Unit]): Nothing = {
     assert(Actor.self == this, "react on channel belonging to other actor")
-    // links
-    if (shouldExit) exit()
-    Scheduler.pendReaction
+    if (shouldExit) exit() // links
     this.synchronized {
-      tick()
+      // first, remove spurious TIMEOUT message from mailbox if any
+      val spurious = mailbox.extractFirst((m: Any) => m == TIMEOUT)
+
       val qel = mailbox.extractFirst((m: Any) => f.isDefinedAt(m))
       if (null eq qel) {
-        waitingFor = f.isDefinedAt
-        TimerThread.requestTimeout(this, f, msec)
-        timeoutPending = true
-        continuation = f
-        isDetached = true
+        if (msec == 0) {
+          if (f.isDefinedAt(TIMEOUT)) {
+            sessions = List(Actor.self)
+            scheduleActor(f, TIMEOUT)
+          }
+          else
+            error("unhandled timeout")
+        }
+        else {
+          waitingFor = f.isDefinedAt
+          val thisActor = this
+          onTimeout = Some(new TimerTask {
+            def run() { thisActor ! TIMEOUT }
+          })
+          Actor.timer.schedule(onTimeout.get, msec)
+          continuation = f
+          isDetached = true
+        }
       } else {
-        sessions = qel.session :: sessions
+        sessions = List(qel.session)
         scheduleActor(f, qel.msg)
       }
       throw new SuspendActorException
@@ -418,31 +573,54 @@ trait Actor extends OutputChannel[Any] {
   /**
    * Sends <code>msg</code> to this actor (asynchronous).
    */
-  def !(msg: Any): Unit = send(msg, Actor.self.getReplyChannel)
+  def !(msg: Any) {
+    send(msg, Actor.self)
+  }
 
-  def forward(msg: Any): Unit = send(msg, Actor.sender.getReplyChannel)
+  /**
+   * Forwards <code>msg</code> to this actor (asynchronous).
+   */
+  def forward(msg: Any) {
+    send(msg, Actor.sender)
+  }
 
   /**
    * Sends <code>msg</code> to this actor and awaits reply
    * (synchronous).
+   *
+   * @param  msg the message to be sent
+   * @return     the reply
    */
   def !?(msg: Any): Any = {
-    val replyChannel = Actor.self.freshReply()
-    this ! msg
-    replyChannel.receive {
+    val replyCh = Actor.self.freshReplyChannel
+    send(msg, replyCh)
+    replyCh.receive {
       case x => x
     }
   }
 
-  def !?(msec: long, msg: Any): Option[Any] = {
-    val replyChannel = Actor.self.freshReply()
-    this ! msg
-    replyChannel.receiveWithin(msec) {
+  /**
+   * Sends <code>msg</code> to this actor and awaits reply
+   * (synchronous) within <code>msec</code> milliseconds.
+   *
+   * @param  msec the time span before timeout
+   * @param  msg  the message to be sent
+   * @return      <code>None</code> in case of timeout, otherwise
+   *              <code>Some(x)</code> where <code>x</code> is the reply
+   */
+  def !?(msec: Long, msg: Any): Option[Any] = {
+    val replyCh = Actor.self.freshReplyChannel
+    send(msg, replyCh)
+    replyCh.receiveWithin(msec) {
       case TIMEOUT => None
       case x => Some(x)
     }
   }
 
+  /**
+   * Sends <code>msg</code> to this actor and immediately
+   * returns a future representing the reply value.
+   */
   def !!(msg: Any): Future[Any] = {
     val ftch = new Channel[Any](Actor.self)
     send(msg, ftch)
@@ -452,6 +630,11 @@ trait Actor extends OutputChannel[Any] {
         else ch.receive {
           case any => value = Some(any); any
         }
+      def respond(k: Any => Unit): Unit =
+ 	if (isSet) k(value.get)
+ 	else ch.react {
+ 	  case any => value = Some(any); k(any)
+ 	}
       def isSet = value match {
         case None => ch.receiveWithin(0) {
           case TIMEOUT => false
@@ -462,15 +645,27 @@ trait Actor extends OutputChannel[Any] {
     }
   }
 
-  def !![a](msg: Any, f: PartialFunction[Any, a]): Future[a] = {
+  /**
+   * Sends <code>msg</code> to this actor and immediately
+   * returns a future representing the reply value.
+   * The reply is post-processed using the partial function
+   * <code>f</code>. This also allows to recover a more
+   * precise type for the reply value.
+   */
+  def !![A](msg: Any, f: PartialFunction[Any, A]): Future[A] = {
     val ftch = new Channel[Any](Actor.self)
     send(msg, ftch)
-    new Future[a](ftch) {
+    new Future[A](ftch) {
       def apply() =
-        if (isSet) value.get
+        if (isSet) value.get.asInstanceOf[A]
         else ch.receive {
-          case any => value = Some(f(any)); value.get
+          case any => value = Some(f(any)); value.get.asInstanceOf[A]
         }
+      def respond(k: A => Unit): Unit =
+ 	if (isSet) k(value.get.asInstanceOf[A])
+ 	else ch.react {
+ 	  case any => value = Some(f(any)); k(value.get.asInstanceOf[A])
+ 	}
       def isSet = value match {
         case None => ch.receiveWithin(0) {
           case TIMEOUT => false
@@ -481,30 +676,37 @@ trait Actor extends OutputChannel[Any] {
     }
   }
 
-  def reply(msg: Any): Unit = session ! msg
+  /**
+   * Replies with <code>msg</code> to the sender.
+   */
+  def reply(msg: Any) {
+    sender ! msg
+  }
 
-  private var rc = new Channel[Any](this)
-  def getReplyChannel = rc
-  def freshReply() = { rc = new Channel[Any]; rc }
+  private var rc: Channel[Any] = null
+  private[actors] def replyChannel = rc
+  private[actors] def freshReplyChannel: Channel[Any] =
+    { rc = new Channel[Any](this); rc }
 
+  /**
+   * Receives the next message from this actor's mailbox.
+   */
   def ? : Any = receive {
     case x => x
   }
 
-  private[actors] def sender: Actor =
-    if (sessions.isEmpty) null
-    else sessions.head.asInstanceOf[Channel[Any]].receiver
+  def sender: OutputChannel[Any] = sessions.head
 
-  private[actors] def session: Channel[Any] =
-    if (sessions.isEmpty) null
-    else sessions.head.asInstanceOf[Channel[Any]]
+  def receiver: Actor = this
 
-  private[actors] var continuation: PartialFunction[Any, Unit] = null
-  private[actors] var timeoutPending = false
+  private var continuation: PartialFunction[Any, Unit] = null
+  private var onTimeout: Option[TimerTask] = None
+  // accessed in Reaction
   private[actors] var isDetached = false
-  private[actors] var isWaiting = false
+  private var isWaiting = false
 
-  private[actors] def scheduleActor(f: PartialFunction[Any, Unit], msg: Any) =
+  // guarded by lock of this
+  protected def scheduleActor(f: PartialFunction[Any, Unit], msg: Any) =
     if ((f eq null) && (continuation eq null)) {
       // do nothing (timeout is handled instead)
     }
@@ -512,17 +714,17 @@ trait Actor extends OutputChannel[Any] {
       val task = new Reaction(this,
                               if (f eq null) continuation else f,
                               msg)
-      Scheduler execute task
+      scheduler execute task
     }
 
-  private[actors] def tick(): Unit =
-    Scheduler tick this
+  private def tick(): Unit =
+    scheduler tick this
 
-  private[actors] var kill = () => {}
+  private[actors] var kill: () => Unit = () => {}
 
-  def suspendActor() {
+  private def suspendActor() {
     isWaiting = true
-    while(isWaiting) {
+    while (isWaiting) {
       try {
         wait()
       } catch {
@@ -533,12 +735,12 @@ trait Actor extends OutputChannel[Any] {
     if (shouldExit) exit()
   }
 
-  def suspendActorFor(msec: long) {
+  private def suspendActorFor(msec: Long) {
     val ts = Platform.currentTime
     var waittime = msec
     var fromExc = false
     isWaiting = true
-    while(isWaiting) {
+    while (isWaiting) {
       try {
         fromExc = false
         wait(waittime)
@@ -557,7 +759,7 @@ trait Actor extends OutputChannel[Any] {
     if (shouldExit) exit()
   }
 
-  def resumeActor() {
+  private def resumeActor() {
     isWaiting = false
     notify()
   }
@@ -565,11 +767,42 @@ trait Actor extends OutputChannel[Any] {
   /**
    * Starts this actor.
    */
-  def start() {
-    Scheduler start new Reaction(this)
+  def start(): Actor = synchronized {
+    // Reset various flags.
+    //
+    // Note that we do *not* reset `trapExit`. The reason is that
+    // users should be able to set the field in the constructor
+    // and before `act` is called.
+
+    exitReason = 'normal
+    exiting = false
+    shouldExit = false
+
+    scheduler execute {
+      ActorGC.newActor(Actor.this)
+      (new Reaction(Actor.this)).run()
+    }
+
+    this
   }
 
-  private[actors] var links: List[Actor] = Nil
+  private def seq[a, b](first: => a, next: => b): Unit = {
+    val s = Actor.self
+    val killNext = s.kill
+    s.kill = () => {
+      s.kill = killNext
+
+      // to avoid stack overflow:
+      // instead of directly executing `next`,
+      // schedule as continuation
+      scheduleActor({ case _ => next }, 1)
+      throw new SuspendActorException
+    }
+    first
+    throw new KillActorException
+  }
+
+  private[actors] var links: List[AbstractActor] = Nil
 
   /**
    * Links <code>self</code> to actor <code>to</code>.
@@ -577,8 +810,11 @@ trait Actor extends OutputChannel[Any] {
    * @param to ...
    * @return   ...
    */
-  def link(to: Actor): Actor = {
-    links = to :: links
+  def link(to: AbstractActor): AbstractActor = {
+    assert(Actor.self == this, "link called on actor different from self")
+    synchronized {
+      links = to :: links
+    }
     to.linkTo(this)
     to
   }
@@ -595,25 +831,27 @@ trait Actor extends OutputChannel[Any] {
     actor
   }
 
-  private[actors] def linkTo(to: Actor) {
+  private[actors] def linkTo(to: AbstractActor) = synchronized {
     links = to :: links
   }
 
   /**
-   Unlinks <code>self</code> from actor <code>from</code>.
+   * Unlinks <code>self</code> from actor <code>from</code>.
    */
-  def unlink(from: Actor) {
-    links = links.remove(from.==)
+  def unlink(from: AbstractActor) {
+    assert(Actor.self == this, "unlink called on actor different from self")
+    synchronized {
+      links = links.remove(from.==)
+    }
     from.unlinkFrom(this)
   }
 
-  private[actors] def unlinkFrom(from: Actor) {
+  private[actors] def unlinkFrom(from: AbstractActor) = synchronized {
     links = links.remove(from.==)
   }
 
   var trapExit = false
   private[actors] var exitReason: AnyRef = 'normal
-  private[actors] var exiting = false
   private[actors] var shouldExit = false
 
   /**
@@ -624,7 +862,7 @@ trait Actor extends OutputChannel[Any] {
    * <p>
    *   For each linked actor <code>a</code> with
    *   <code>trapExit</code> set to <code>true</code>, send message
-   *   <code>{'EXIT, self, reason}</code> to <code>a</code>.
+   *   <code>Exit(self, reason)</code> to <code>a</code>.
    * </p>
    * <p>
    *   For each linked actor <code>a</code> with
@@ -634,24 +872,27 @@ trait Actor extends OutputChannel[Any] {
    * </p>
    */
   def exit(reason: AnyRef): Nothing = {
-    if (reason == 'normal) kill()
-    // links
-    if (!links.isEmpty) {
-      exitReason = reason
-      exitLinked()
-    }
-    throw new ExitActorException
+    exitReason = reason
+    exit()
   }
 
-  def exit(): Nothing = exit('normal)
+  /**
+   * Terminates with exit reason <code>'normal</code>.
+   */
+  def exit(): Nothing = {
+    // links
+    if (!links.isEmpty)
+      exitLinked()
+    throw new ExitActorException
+  }
 
   // Assume !links.isEmpty
   private[actors] def exitLinked() {
     exiting = true
     // remove this from links
-    links = links.remove(this.==)
+    val mylinks = links.remove(this.==)
     // exit linked processes
-    links.foreach((linked: Actor) => {
+    mylinks.foreach((linked: AbstractActor) => {
       unlink(linked)
       if (!linked.exiting)
         linked.exit(this, exitReason)
@@ -661,21 +902,13 @@ trait Actor extends OutputChannel[Any] {
   // Assume !links.isEmpty
   private[actors] def exitLinked(reason: AnyRef) {
     exitReason = reason
-    exiting = true
-    // remove this from links
-    links = links.remove(this.==)
-    // exit linked processes
-    links.foreach((linked: Actor) => {
-      unlink(linked)
-      if (!linked.exiting)
-        linked.exit(this, exitReason)
-    })
+    exitLinked()
   }
 
   // Assume !this.exiting
-  private[actors] def exit(from: Actor, reason: AnyRef) {
+  private[actors] def exit(from: AbstractActor, reason: AnyRef) {
     if (trapExit) {
-      this ! Triple('EXIT, from, reason)
+      this ! Exit(from, reason)
     }
     else if (reason != 'normal)
       this.synchronized {
@@ -691,28 +924,35 @@ trait Actor extends OutputChannel[Any] {
 }
 
 
-/**
- * <p>This object is used as the timeout pattern in
- * <code>receiveWithin</code> and <code>reactWithin</code>.
- * </p>
- * <p>The following example demonstrates its usage:</p><pre>
- * receiveWithin(500) {
- *   case {x, y}  => ...
- *   case TIMEOUT => ...
- * }
- * </pre>
+/** <p>
+ *    This object is used as the timeout pattern in
+ *    <a href="Actor.html#receiveWithin(Long)" target="contentFrame">
+ *    <code>receiveWithin</code></a> and
+ *    <a href="Actor.html#reactWithin(Long)" target="contentFrame">
+ *    <code>reactWithin</code></a>.
+ *  </p>
+ *  <p>
+ *    The following example demonstrates its usage:
+ *  </p><pre>
+ *    receiveWithin(500) {
+ *      <b>case</b> (x, y) <b>=&gt;</b> ...
+ *      <b>case</b> TIMEOUT <b>=&gt;</b> ...
+ *    }</pre>
  *
- * @version 0.9.4
- * @author Philipp Haller
+ *  @version 0.9.8
+ *  @author Philipp Haller
  */
 case object TIMEOUT
 
 
-/**
- * <p>This class is used to manage control flow of actor
- * executions.</p>
+case class Exit(from: AbstractActor, reason: AnyRef)
+
+/** <p>
+ *    This class is used to manage control flow of actor
+ *    executions.
+ *  </p>
  *
- * @version 0.9.4
+ * @version 0.9.8
  * @author Philipp Haller
  */
 private[actors] class SuspendActorException extends Throwable {
