@@ -1,12 +1,12 @@
 /*                     __                                               *\
 **     ________ ___   / /  ___     Scala API                            **
-**    / __/ __// _ | / /  / _ |    (c) 2005-2007, LAMP/EPFL             **
+**    / __/ __// _ | / /  / _ |    (c) 2005-2009, LAMP/EPFL             **
 **  __\ \/ /__/ __ |/ /__/ __ |                                         **
 ** /____/\___/_/ |_/____/_/ | |                                         **
 **                          |/                                          **
 \*                                                                      */
 
-// $Id: Reaction.scala 15687 2008-08-03 21:15:01Z phaller $
+// $Id: Reaction.scala 17381 2009-03-25 10:38:46Z phaller $
 
 
 package scala.actors
@@ -103,6 +103,9 @@ class Reaction extends Runnable {
       }
     } finally {
       Actor.tl.set(saved)
+      this.a = null
+      this.f = null
+      this.msg = null
     }
   }
 
