@@ -1,12 +1,12 @@
 /*                     __                                               *\
 **     ________ ___   / /  ___     Scala API                            **
-**    / __/ __// _ | / /  / _ |    (c) 2005-2007, LAMP/EPFL             **
+**    / __/ __// _ | / /  / _ |    (c) 2005-2009, LAMP/EPFL             **
 **  __\ \/ /__/ __ |/ /__/ __ |    http://scala-lang.org/               **
 ** /____/\___/_/ |_/____/_/ | |                                         **
 **                          |/                                          **
 \*                                                                      */
 
-// $Id: RemoteActor.scala 10984 2007-05-10 12:04:55Z michelou $
+// $Id: RemoteActor.scala 17018 2009-02-02 21:36:04Z phaller $
 
 
 package scala.actors.remote
@@ -32,38 +32,67 @@ package scala.actors.remote
  *  </p><pre>
  *  actor {
  *    // ...
- *    <b>val</b> c = select(TcpNode("127.0.0.1", 9010), 'myName)
+ *    <b>val</b> c = select(Node("127.0.0.1", 9010), 'myName)
  *    c ! msg
  *    // ...
  *  }
  *  </pre>
  *
- *  @author Philipp Haller
+ * @version 0.9.18
+ * @author Philipp Haller
  */
 object RemoteActor {
 
   private val kernels = new scala.collection.mutable.HashMap[Actor, NetKernel]
 
+  /* If set to <code>null</code> (default), the default class loader
+   * of <code>java.io.ObjectInputStream</code> is used for deserializing
+   * objects sent as messages.
+   */
+  private var cl: ClassLoader = null
+
+  def classLoader: ClassLoader = cl
+  def classLoader_=(x: ClassLoader) { cl = x }
+
   /**
    * Makes <code>self</code> remotely accessible on TCP port
    * <code>port</code>.
    */
-  def alive(port: int) {
-    val serv = new TcpService(port)
-    serv.start()
-    kernels += Actor.self -> serv.kernel
+  def alive(port: Int): Unit = synchronized {
+    createKernelOnPort(port)
+  }
+
+  def createKernelOnPort(port: Int): NetKernel = {
+    val serv = TcpService(port, cl)
+    val kern = serv.kernel
+    val s = Actor.self
+    kernels += Pair(s, kern)
+
+    ActorGC.onTerminate(s) {
+      Debug.info("alive actor "+s+" terminated")
+      // remove mapping for `s`
+      kernels -= s
+      // terminate `kern` when it does
+      // not appear as value any more
+      if (!kernels.values.contains(kern)) {
+        Debug.info("terminating "+kern)
+        // terminate NetKernel
+        kern.terminate()
+      }
+    }
+
+    kern
   }
 
   /**
    * Registers <code>a</code> under <code>name</code> on this
    * node.
    */
-  def register(name: Symbol, a: Actor) {
+  def register(name: Symbol, a: Actor): Unit = synchronized {
     val kernel = kernels.get(Actor.self) match {
       case None =>
-        val serv = new TcpService(TcpService.generatePort)
-        serv.start()
-        kernels += Actor.self -> serv.kernel
+        val serv = TcpService(TcpService.generatePort, cl)
+        kernels += Pair(Actor.self, serv.kernel)
         serv.kernel
       case Some(k) =>
         k
@@ -71,38 +100,34 @@ object RemoteActor {
     kernel.register(name, a)
   }
 
+  private def selfKernel = kernels.get(Actor.self) match {
+    case None =>
+      // establish remotely accessible
+      // return path (sender)
+      createKernelOnPort(TcpService.generatePort)
+    case Some(k) =>
+      k
+  }
+
   /**
    * Returns (a proxy for) the actor registered under
    * <code>name</code> on <code>node</code>.
    */
-  def select(node: Node, sym: Symbol): Actor =
-    new Actor {
-      def act() {}
-      override def !(msg: Any): Unit = msg match {
-        case a: AnyRef =>
-          // establish remotely accessible
-          // return path (sender)
-          val kernel = kernels.get(Actor.self) match {
-            case None =>
-              val serv = new TcpService(TcpService.generatePort)
-              serv.start()
-              kernels += Actor.self -> serv.kernel
-              serv.kernel
-            case Some(k) =>
-              k
-          }
-          kernel.send(node, sym, a)
+  def select(node: Node, sym: Symbol): AbstractActor = synchronized {
+    selfKernel.getOrCreateProxy(node, sym)
+  }
 
-        case other =>
-          error("Cannot send non-AnyRef value remotely.")
-      }
-    }
+  def someKernel: NetKernel =
+    kernels.values.next
 }
 
 
 /**
  * This class represents a machine node on a TCP network.
  *
+ * @param address the host name, or <code>null</code> for the loopback address.
+ * @param port    the port number.
+ *
  * @author Philipp Haller
  */
 case class Node(address: String, port: Int)
