@@ -1,22 +1,22 @@
 /*                     __                                               *\
 **     ________ ___   / /  ___     Scala API                            **
-**    / __/ __// _ | / /  / _ |    (c) 2003-2007, LAMP/EPFL             **
-**  __\ \/ /__/ __ |/ /__/ __ |                                         **
+**    / __/ __// _ | / /  / _ |    (c) 2003-2009, LAMP/EPFL             **
+**  __\ \/ /__/ __ |/ /__/ __ |    http://scala-lang.org/               **
 ** /____/\___/_/ |_/____/_/ | |                                         **
 **                          |/                                          **
 \*                                                                      */
 
-// $Id: ListSet.scala 9554 2007-01-04 16:30:16 +0000 (Thu, 04 Jan 2007) odersky $
+// $Id: Set1.scala 16894 2009-01-13 13:09:41Z cunei $
 
 
 
 package scala.collection.immutable
 
-/** This class implements empty immutable maps
+/** This class implements immutable sets with exactly one element.
  *  @author  Martin Oderskty
  *  @version 1.0, 019/01/2007
  */
-[serializable]
+@serializable
 class Set1[A](elem1: A) extends Set[A] {
 
   def empty[C]: Set[C] = new EmptySet[C]
