@@ -1,12 +1,12 @@
 /*                     __                                               *\
 **     ________ ___   / /  ___     Scala API                            **
-**    / __/ __// _ | / /  / _ |    (c) 2003-2006, LAMP/EPFL             **
+**    / __/ __// _ | / /  / _ |    (c) 2003-2009, LAMP/EPFL             **
 **  __\ \/ /__/ __ |/ /__/ __ |                                         **
 ** /____/\___/_/ |_/____/_/ | |                                         **
 **                          |/                                          **
 \*                                                                      */
 
-// $Id: ObservableMap.scala 9980 2007-02-13 22:06:58Z odersky $
+// $Id: ObservableMap.scala 16894 2009-01-13 13:09:41Z cunei $
 
 
 package scala.collection.mutable
@@ -21,21 +21,21 @@ package scala.collection.mutable
  *  @author  Martin Odersky
  *  @version 2.0, 31/12/2006
  */
-trait ObservableMap[A, B, This <: ObservableMap[A, B, This]] requires This
+trait ObservableMap[A, B, This <: ObservableMap[A, B, This]] 
       extends Map[A, B]
-      with Publisher[Message[Pair[A, B]]
+      with Publisher[Message[(A, B)]
       with Undoable, This]
-{
+{ self: This =>
 
   abstract override def update(key: A, value: B): Unit = get(key) match {
     case None =>
       super.update(key, value)
-      publish(new Include(Pair(key, value)) with Undoable {
+      publish(new Include((key, value)) with Undoable {
         def undo = -=(key)
       })
     case Some(old) =>
       super.update(key, value)
-      publish(new Update(Pair(key, value)) with Undoable {
+      publish(new Update((key, value)) with Undoable {
         def undo = update(key, old)
       })
   }
@@ -44,7 +44,7 @@ trait ObservableMap[A, B, This <: ObservableMap[A, B, This]] requires This
     case None =>
     case Some(old) =>
       super.-=(key)
-      publish(new Remove(Pair(key, old)) with Undoable {
+      publish(new Remove((key, old)) with Undoable {
         def undo = update(key, old)
       })
   }
