@@ -1,12 +1,12 @@
 /*                     __                                               *\
 **     ________ ___   / /  ___     Scala API                            **
-**    / __/ __// _ | / /  / _ |    (c) 2003-2006, LAMP/EPFL             **
-**  __\ \/ /__/ __ |/ /__/ __ |                                         **
+**    / __/ __// _ | / /  / _ |    (c) 2003-2009, LAMP/EPFL             **
+**  __\ \/ /__/ __ |/ /__/ __ |    http://scala-lang.org/               **
 ** /____/\___/_/ |_/____/_/ | |                                         **
 **                          |/                                          **
 \*                                                                      */
 
-// $Id: Benchmark.scala 9492 2006-12-10 00:35:37Z emir $
+// $Id: Benchmark.scala 16894 2009-01-13 13:09:41Z cunei $
 
 
 package scala.testing
@@ -14,17 +14,17 @@ package scala.testing
 
 import compat.Platform
 
-/** <code>Benchmark</code> can be used to quickly turn an existing
- *  class into a benchmark. Here is a short example:
- *
- *  <pre>
+/** <p>
+ *    <code>Benchmark</code> can be used to quickly turn an existing
+ *    class into a benchmark. Here is a short example:
+ *  </p><pre>
  *  <b>object</b> sort1 <b>extends</b> Sorter <b>with</b> Benchmark {
  *    <b>def</b> run = sort(List.range(1, 1000))
  *  }
  *  </pre>
  *  <p>
- *    The run method has to be defined by the user, who will perform
- *    the timed operation there.
+ *    The <code>run</code> method has to be defined by the user, who
+ *    will perform the timed operation there.
  *    Run the benchmark as follows:
  *  </p>
  *  <pre>
@@ -34,11 +34,13 @@ import compat.Platform
  *    This will run the benchmark 5 times and log the execution times in
  *    a file called <code>times.log</code>
  *  </p>
+ *
+ *  @author Iulian Dragos, Burak Emir
  */
 trait Benchmark {
 
   /** this method should be implemented by the concrete benchmark */
-  def run: Unit
+  def run()
 
   var multiplier = 1
 
@@ -50,11 +52,11 @@ trait Benchmark {
    *  @return        ...
    */
   def runBenchmark(noTimes: Int): List[Long] =
-    for (val i <- List.range(1, noTimes + 1)) yield {
+    for (i <- List.range(1, noTimes + 1)) yield {
       val startTime = Platform.currentTime
-      var i = 0; while(i < multiplier) {
-        run
-        i = i + 1
+      var i = 0; while (i < multiplier) {
+        run()
+        i += 1
       }
       val stopTime = Platform.currentTime
       Platform.collectGarbage
@@ -62,18 +64,26 @@ trait Benchmark {
       stopTime - startTime
     }
 
-  /**
-   * The entry point. It takes two arguments: the number of
-   * consecutive runs, and the name of a log file where to
-   * append the times.
+  /** a string that is written at the beginning of the output line
+   *   that contains the timings. By default, this is the class name.
    */
-  def main(args: Array[String]): Unit = {
+  def prefix: String = getClass().getName()
+
+  /**
+   * The entry point. It takes two arguments (n), (name) 
+   *  and an optional argument multiplier (mult).
+   *  (n) is the number of consecutive runs, (name) the name 
+   *  of a log file where to append the times.
+   *  if (mult) is present, the same thing is repeated (mult)
+   *  times.
+   */
+  def main(args: Array[String]) {
     if (args.length > 1) {
       val logFile = new java.io.FileWriter(args(1), true) // append, not overwrite
-      if(args.length >= 3)
+      if (args.length >= 3)
          multiplier = args(2).toInt
-      logFile.write(getClass().getName())
-      for (val t <- runBenchmark(args(0).toInt))
+      logFile.write(prefix)
+      for (t <- runBenchmark(args(0).toInt))
         logFile.write("\t\t" + t)
 
       logFile.write(Platform.EOL)
