@@ -1,18 +1,16 @@
 /*                     __                                               *\
 **     ________ ___   / /  ___     Scala API                            **
-**    / __/ __// _ | / /  / _ |    (c) 2003-2006, LAMP/EPFL             **
-**  __\ \/ /__/ __ |/ /__/ __ |                                         **
+**    / __/ __// _ | / /  / _ |    (c) 2003-2009, LAMP/EPFL             **
+**  __\ \/ /__/ __ |/ /__/ __ |    http://scala-lang.org/               **
 ** /____/\___/_/ |_/____/_/ | |                                         **
 **                          |/                                          **
 \*                                                                      */
 
-// $Id: Node.scala 9454 2006-12-03 00:32:39Z emir $
+// $Id: Node.scala 16894 2009-01-13 13:09:41Z cunei $
 
 
 package scala.xml
 
-import compat.StringBuilder
-
 /**
  * This object provides methods ...
  *
@@ -27,6 +25,8 @@ object Node {
   /** the empty namespace */
   val EmptyNamespace = ""
 
+  def unapplySeq(n: Node) = Some(Tuple3(n.label, n.attributes, n.child))
+
 }
 
 /**
@@ -49,8 +49,9 @@ abstract class Node extends NodeSeq {
   def typeTag$: Int = 0
 
   /**
-   *  method returning the namespace bindings of this node. by default, this is TopScope,
-   *  which means there are no namespace bindings except the predefined one for "xml".
+   *  method returning the namespace bindings of this node. by default, this
+   *  is TopScope, which means there are no namespace bindings except the
+   *  predefined one for "xml".
    */
   def scope: NamespaceBinding = TopScope
 
@@ -88,11 +89,13 @@ abstract class Node extends NodeSeq {
    * @return value of <code>PrefixedAttribute</code> with given namespace
    *         and given key, otherwise <code>null</code>.
    */
-  final def attribute(uri: String, key: String): Option[Seq[Node]] = attributes.get(uri, this, key)
+  final def attribute(uri: String, key: String): Option[Seq[Node]] =
+    attributes.get(uri, this, key)
 
   /**
-   * Returns attribute meaning all attributes of this node, prefixed and unprefixed, 
-   * in no particular order. In class Node, this defaults to Null (the empty attribute list).
+   * Returns attribute meaning all attributes of this node, prefixed and
+   * unprefixed, in no particular order. In class <code>Node</code>, this
+   * defaults to <code>Null</code> (the empty attribute list).
    *
    * @return all attributes of this node
    */
@@ -135,11 +138,14 @@ abstract class Node extends NodeSeq {
     case _ => false
   }
 
-  /**
-   *  Returns a hashcode. A standard implementation of hashcodes is obtained by calling 
-   *  Utility.hashCode(pre, label, attributes.hashCode(), child); 
+  /** <p>
+   *    Returns a hashcode. The default implementation here calls only
+   *    super.hashcode (which is the same as for objects). A more useful
+   *    implementation can be invoked by calling 
+   *  <code>Utility.hashCode(pre, label, attributes.hashCode(), child)</code>.
+   *  </p>
    */
-  override def hashCode(): Int
+  override def hashCode(): Int = super.hashCode
 
   // implementations of NodeSeq methods
 
@@ -170,12 +176,12 @@ abstract class Node extends NodeSeq {
    * @param sb ...
    * @return   ...
    */
-  def nameToString(sb: StringBuilder): StringBuilder  = {
+  def nameToString(sb: StringBuilder): StringBuilder = {
     if (null != prefix) {
       sb.append(prefix)
       sb.append(':')
     }
-    sb.append(label);
+    sb.append(label)
   }
 
   /**
@@ -187,7 +193,12 @@ abstract class Node extends NodeSeq {
    * Returns a text representation of this node. Note that this is not equivalent to
    * the XPath node-test called text(), it is rather an implementation of the
    * XPath function string()
+   *  Martin to Burak: to do: if you make this method abstract, the compiler will now
+   *  complain if there's no implementation in a subclass. Is this what we want? Note that
+   *  this would break doc/DocGenator and doc/ModelToXML, with an error message like:
+doc\DocGenerator.scala:1219: error: object creation impossible, since there is a deferred declaration of method text in class Node of type => String which is not implemented in a subclass
+    new SpecialNode {
+    ^
    */
-  override def text: String
-
+  override def text: String = super.text
 }
