@@ -1,22 +1,23 @@
 /*                     __                                               *\
 **     ________ ___   / /  ___     Scala API                            **
-**    / __/ __// _ | / /  / _ |    (c) 2003-2006, LAMP/EPFL             **
-**  __\ \/ /__/ __ |/ /__/ __ |                                         **
+**    / __/ __// _ | / /  / _ |    (c) 2003-2009, LAMP/EPFL             **
+**  __\ \/ /__/ __ |/ /__/ __ |    http://scala-lang.org/               **
 ** /____/\___/_/ |_/____/_/ | |                                         **
 **                          |/                                          **
 \*                                                                      */
 
-// $Id: Text.scala 8097 2006-07-11 15:15:02Z emir $
+// $Id: Text.scala 16894 2009-01-13 13:09:41Z cunei $
 
 
 package scala.xml
 
-import scala.runtime.compat.StringBuilder
 
-/** an XML node for text (PCDATA). Used in both non-bound and bound XML 
- *  representations
- * @author Burak Emir
- * @param text the text contained in this node, may not be null.
+/** The class <code>Text</code> implements an XML node for text (PCDATA).
+ *  It is used in both non-bound and bound XML representations.
+ *
+ *  @author Burak Emir
+ *
+ *  @param text the text contained in this node, may not be null.
  */
 case class Text(_data: String) extends Atom[String](_data) {
 
@@ -24,13 +25,19 @@ case class Text(_data: String) extends Atom[String](_data) {
     throw new java.lang.NullPointerException("tried to construct Text with null")
 
   final override def equals(x: Any) = x match {
-    case s:String  => s.equals(data.toString())
+    case s:String  => s == data
     case s:Text    => data == s.data
+    case s:Atom[_] => data == s.data
     case _ => false
   }
 
-  /** returns text, with some characters escaped according to XML spec */
+  /** Returns text, with some characters escaped according to the XML
+   *  specification.
+   *
+   *  @param  sb ...
+   *  @return ... 
+   */
   override def toString(sb: StringBuilder) =
-    Utility.escape(data.toString(), sb)
+    Utility.escape(data, sb)
 
 }
