@@ -1,12 +1,12 @@
 /*                     __                                               *\
 **     ________ ___   / /  ___     Scala API                            **
-**    / __/ __// _ | / /  / _ |    (c) 2003-2006, LAMP/EPFL             **
+**    / __/ __// _ | / /  / _ |    (c) 2003-2009, LAMP/EPFL             **
 **  __\ \/ /__/ __ |/ /__/ __ |                                         **
 ** /____/\___/_/ |_/____/_/ | |                                         **
 **                          |/                                          **
 \*                                                                      */
 
-// $Id: NoBindingFactoryAdapter.scala 6904 2006-03-25 14:47:10Z emir $
+// $Id: NoBindingFactoryAdapter.scala 16894 2009-01-13 13:09:41Z cunei $
 
 
 package scala.xml.parsing;
@@ -48,7 +48,12 @@ class NoBindingFactoryAdapter extends FactoryAdapter with NodeFactory[Elem] {
   */
   def createText( text:String ) = 
     Text( text );
-  
+
+  /** create a processing instruction
+  */
+  def createProcInstr(target: String, data: String) = 
+    makeProcInstr(target, data)
+    
   /** loads an XML document, returning a Symbol node.
   */
   override def loadXML( source:InputSource ):Elem = 
